"""Outset Python Package"""
from __future__ import annotations

__version__ = "0.0.0""-rc61-post1"

from .draw_outset_ import draw_outset
from .mark_arrow_ import mark_arrow
from .mark_inlaid_asterisk_ import mark_inlaid_asterisk
from .mark_magnifying_glass_ import mark_magnifying_glass
from .outsetplot_ import outsetplot

__all__ = [
    "draw_outset",
    "mark_arrow",
    "mark_inlaid_asterisk",
    "mark_magnifying_glass",
    "outsetplot",
]



import typing

import frozendict
import numpy as np
from matplotlib import axes as mpl_axes
from matplotlib import pyplot as plt

from ._auxlib.draw_callout_ import draw_callout
from ._auxlib.draw_frame_ import draw_frame
from .mark_magnifying_glass_ import mark_magnifying_glass


def draw_outset(
    frame_xlim: typing.Tuple[float, float],
    frame_ylim: typing.Tuple[float, float],
    ax: typing.Optional[mpl_axes.Axes] = None,
    *,
    color: str = "blue",
    frame_facealpha: float = 0.1,
    frame_linewidth: float = 1,
    clip_on: bool = False,
    hide_outer_spines: bool = True,
    mark_glyph: typing.Optional[typing.Callable] = mark_magnifying_glass,
    mark_glyph_kwargs: typing.Dict = frozendict.frozendict(),
    mark_retract: float = 0.1,
    frame_inner_pad: float = 0.0,
    leader_linestyle: str = ":",
    leader_linewidth: int = 2,
    leader_stretch: float = 0.1,
    zorder: float = 0,
) -> typing.Tuple[mpl_axes.Axes, typing.Tuple[float, float, float, float]]:
    """Mark a rectangular region as outset, framing it and adding a
    "zoom"-effect callout up and to the right.

    The callout is capped by a customizable glyph, default as a magnifying
    glass.

    Parameters
    ----------
    frame_xlim : Tuple[float, float]
        The x-limits (xmin, xmax) of the area to be marked outset.
    frame_ylim : Tuple[float, float]
        The y-limits (ymin, ymax) of the area to be marked outset.
    ax : matplotlib.axes.Axes, optional
        The axes object on which to draw the outset.

        Defaults to `plt.gca()`.
    color : str, default "blue"
        Color for the frame's edge and the lines of the zoom indication.
    frame_facealpha : float, default 0.1
        Alpha value for the frame's fill color, controlling its transparency.
    frame_linewidth : float, default 1
        Line width of the frame's edge.
    clip_on : bool, default False
        Determines if drawing elements should be clipped to the axes bounding
        box.
    hide_outer_spines : bool, default True
        If True, hides the top and right spines of the axes.
    mark_glyph : Optional[Callable], optional
        A callable to draw a glyph at the end of the callout.

        Defaults to a magnifying glass. Outset also provides implementations
        for arrow, asterisk, and letter/number glyphs.
    mark_glyph_kwargs : Dict, default frozendict.frozendict()
        Keyword arguments for the mark_glyph callable.
    mark_retract : float, default 0.1
        Retraction factor for the glyph placement from the outer vertex of the
        callout.
    frame_inner_pad : float, default 0.0
        Padding factor for the inner margin of the frame.
    leader_linestyle : str, default ":"
        Line style for the zoom indication (e.g., solid, dashed, dotted).
    leader_linewidth : int, default 2
        Line width for the zoom indication's edges.
    leader_stretch : float, default 0.1
        Stretch factor for the callout leader extending from the frame.

        Set `leader_stretch` 0 to collapse away the leader.
    zorder : float, default 0
        Z-order for layering plot elements; higher values are drawn on top.

    Returns
    -------
    Tuple[mpl_axes.Axes, Tuple[float, float, float, float]]
        The axes object with the outset and annotations added.
    """
    if ax is None:
        ax = plt.gca()

    pad_x = (ax.get_xlim()[1] - ax.get_xlim()[0]) * frame_inner_pad
    frame_xlim = np.array(frame_xlim) + np.array([-pad_x, pad_x])
    pad_y = (ax.get_ylim()[1] - ax.get_ylim()[0]) * frame_inner_pad
    frame_ylim = np.array(frame_ylim) + np.array([-pad_y, pad_y])

    # tweak zorder to ensure multiple outset annotations layer properly
    ax_width = ax.get_xlim()[1] - ax.get_xlim()[0]
    ax_height = ax.get_ylim()[1] - ax.get_ylim()[0]
    ax_diag = np.sqrt(ax_width**2 + ax_height**2)
    upper_right_drop_x = ax.get_ylim()[1] - frame_xlim[1]
    upper_right_drop_y = ax.get_ylim()[1] - frame_ylim[1]
    upper_right_drop = np.sqrt(
        upper_right_drop_x**2 + upper_right_drop_y**2
    )
    zorder += 0.25 * upper_right_drop / ax_diag

    # Frame outset region
    ###########################################################################
    draw_frame(
        frame_xlim,
        frame_ylim,
        ax=ax,
        clip_on=clip_on,
        edgecolor=color,
        facecolor=(color, frame_facealpha),
        linewidth=frame_linewidth,
        zorder=zorder,
    )

    # Draw callout
    ###########################################################################
    draw_callout(
        frame_xlim,
        frame_ylim,
        ax,
        color=color,
        clip_on=clip_on,
        linewidth=leader_linewidth,
        linestyle=leader_linestyle,
        mark_glyph=mark_glyph,
        mark_glyph_kwargs=mark_glyph_kwargs,
        mark_retract=mark_retract,
        leader_stretch=leader_stretch,
        zorder=zorder,
    )

    # Finalize
    ###########################################################################
    ax.set_axisbelow(True)  # ensure annotations above if outside bounds
    if hide_outer_spines:
        ax.spines[["right", "top"]].set_visible(False)

    return ax, (*frame_xlim, *frame_ylim)  # unpacks into tuple ctor



import typing

import frozendict
import numpy as np
from matplotlib import axes as mpl_axes
from matplotlib import colors as mpl_colors
from matplotlib import patches as mpl_patches
from matplotlib import pyplot as plt

from .compose_callout_leader_ import compose_callout_leader
from .get_vertices_extent_ import get_vertices_extent
from .make_radial_gradient_ import make_radial_gradient


def draw_callout(
    frame_xlim: typing.Tuple[float, float],
    frame_ylim: typing.Tuple[float, float],
    ax: typing.Optional[mpl_axes.Axes] = None,
    *,
    color: str = "blue",
    clip_on: bool = False,
    mark_glyph: typing.Optional[typing.Callable] = None,
    mark_glyph_kwargs: typing.Dict = frozendict.frozendict(),
    mark_retract: float = 0.1,
    leader_stretch: float = 0.1,
    zorder: float = 0,
    **kwargs,
) -> mpl_axes.Axes:
    """Annotate a rectangular region with a flyaway "zoom" indication upwards
    and to the right.

    Parameters
    ----------
    frame_xlim : Tuple[float, float]
        The x-limits (xmin, xmax) of the rectangular region to be annotated.
    frame_ylim : Tuple[float, float]
        The y-limits (ymin, ymax) of the rectangular region to be annotated.
    ax : matplotlib.axes.Axes, optional
        The axes object on which to draw the callout.

        Defaults to `plt.gca()`.
    color : str, default "blue"
        Color for the callout leader and glyph.
    clip_on : bool, default False
        Determines if drawing elements should be clipped to the axes bounding box.
    mark_glyph : Optional[Callable], optional
        A callable to draw a glyph at the outer vertex of the callout leader.

        if None, no glyph is drawn.
    mark_retract : float, default 0.1
        Fraction to pull back glyph from outer vertex of the callout.
    leader_stretch : float, default 0.1
        Scale of callout leader relative to axis viewport.
    zorder : float, default 0
        Influences layer order of plot, with higher values in front.
    **kwargs
        Additional keyword arguments for matplotlib Polygon used in the callout.

    Returns
    -------
    mpl_axes.Axes
        The modified matplotlib axes object with the drawing elements added.
    """
    if ax is None:
        ax = plt.gca()

    # Draw callout leader
    ###########################################################################
    leader_vertices = compose_callout_leader(
        frame_xlim,
        frame_ylim,
        ax.get_xlim(),
        ax.get_ylim(),
        stretch=leader_stretch,
    )

    # ... outline
    leader_patch = mpl_patches.Polygon(
        leader_vertices,  # w/ upper right corner
        closed=True,
        facecolor="none",
        edgecolor=color,
        zorder=zorder,
        clip_on=clip_on,
        **kwargs,
    )
    ax.add_patch(leader_patch)

    # ... gradient fill, clipped insideleader_polygon
    img = ax.imshow(
        make_radial_gradient(),
        aspect="auto",
        cmap=mpl_colors.LinearSegmentedColormap.from_list(
            "gradient",
            ["white", color],
        ),
        extent=get_vertices_extent(leader_vertices),
        interpolation="nearest",
        zorder=zorder,
    )
    if not clip_on:
        img.set_clip_box(ax.bbox.shrunk(10, 10))  # grow axis clipping box
    img.set_clip_path(leader_patch)

    # Draw callout glyph
    ###########################################################################
    frame_upper_right = np.array([frame_xlim[1], frame_ylim[1]])
    leader_outer_vertex = tuple(np.max(np.array(leader_vertices), axis=0))
    assert len(leader_outer_vertex) == 2
    assert all(np.array(leader_outer_vertex) >= np.array(frame_upper_right))

    mark_coordinates = np.average(
        np.array([leader_outer_vertex, frame_upper_right]),
        weights=np.array([1.0 - mark_retract, mark_retract]),
        axis=0,
    )
    if mark_glyph is not None:
        mark_glyph(
            *mark_coordinates,
            ax=ax,
            **{
                "color": color,
                "clip_on": clip_on,
                "zorder": zorder,
                **mark_glyph_kwargs,
            },
        )



import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from matplotlib.testing import decorators as mpl_testing_decorators

from outset import draw_outset, outsetplot

# Sample data for testing
data = pd.DataFrame(
    {"x": [1, 2, 3, 4], "y": [1, 3, 2, 1], "hue": ["A", "B", "A", "B"]}
)


@mpl_testing_decorators.check_figures_equal()
def test_outsetplot_vs_draw_outset(fig_test: plt.Figure, fig_ref: plt.Figure):
    # Using outsetplot
    ax_test = fig_test.subplots()
    outsetplot(data, x="x", y="y", ax=ax_test)

    # Expected output using draw_outset
    ax_ref = fig_ref.subplots()
    draw_outset(
        (1, 4),
        (1, 3),
        ax=ax_ref,
        frame_inner_pad=0.1,
        color=sns.color_palette()[0],
    )


@mpl_testing_decorators.check_figures_equal()
def test_outsetplot_vs_draw_outset_hue(
    fig_test: plt.Figure, fig_ref: plt.Figure
):
    palette = ["green", "red"]
    # Using outsetplot
    ax_test = fig_test.subplots()
    outsetplot(
        data,
        x="x",
        y="y",
        hue="hue",
        hue_order=["A", "B"],
        ax=ax_test,
        frame_inner_pad=0.0,
        palette=palette,
    )
    fig_test.savefig("/tmp/a.png")

    # Expected output using draw_outset
    ax_ref = fig_ref.subplots()
    draw_outset(
        (1, 3),
        (1, 2),
        ax=ax_ref,
        color=palette[0],
        frame_inner_pad=0.0,
    )
    draw_outset(
        (2, 4),
        (1, 3),
        ax=ax_ref,
        color=palette[1],
        frame_inner_pad=0.0,
    )
    fig_ref.savefig("/tmp/b.png")


def test_outsetplot_one():
    plt.clf()
    # Create a sample dataframe
    data = pd.DataFrame({"x": [0.5], "y": [1]})
    sns.scatterplot(data=data, x="x", y="y")
    outsetplot(
        data=data,
        x="x",
        y="y",
        frame_inner_pad=0.2,
    )
    outpath = "/tmp/test_outsetplot_one.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")


def test_outsetplot_several():
    plt.clf()
    _fig, ax = plt.subplots(figsize=(6, 4))
    ax.set_xlim(0, 4)
    ax.set_ylim(0, 4)
    # Create sample data with a 'hue' column for grouping
    data = pd.DataFrame(
        {
            "x": [0.825, 3.1, 0.5, 1.3, 2.2, 2],
            "y": [1.2, 0.8, 3.5, 2.3, 1.1, 3.7],
            "hue": ["group1", "group1", "group2", "group2", "group3", "group3"],
        }
    )

    sns.scatterplot(data=data, x="x", y="y", hue="hue", ax=ax)
    outsetplot(data=data, x="x", y="y", hue="hue", ax=ax, leader_stretch=0.0)

    outpath = "/tmp/test_outsetplot_several.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")



import typing

from matplotlib import axes as mpl_axes
from matplotlib import patches as mpl_patches
from matplotlib import pyplot as plt


def draw_frame(
    frame_xlim: typing.Tuple[float, float],
    frame_ylim: typing.Tuple[float, float],
    ax: typing.Optional[mpl_axes.Axes] = None,
    **kwargs,
) -> mpl_axes.Axes:
    """Mark rectangular region as outset.

    Parameters
    ----------
    frame_xlim : Tuple[float, float]
        The x-limits of the rectangular frame in the form (xmin, xmax).
    frame_ylim : Tuple[float, float]
        The y-limits of the rectangular frame in the form (ymin, ymax).
    ax : matplotlib.axes.Axes, optional
        The axes object on which to draw. If None, the current active axes will
        be used.
    **kwargs
        Additional keyword arguments leader matplotlib `Rectangle`.

    Returns
    -------
    mpl_axes.Axes
        The modified matplotlib axes object with the drawing elements added.
    """
    if ax is None:
        ax = plt.gca()

    frame_patch = mpl_patches.Rectangle(
        (frame_xlim[0], frame_ylim[0]),  # lower left corner
        frame_xlim[1] - frame_xlim[0],  # width
        frame_ylim[1] - frame_ylim[0],  # height
        **kwargs,
    )
    ax.add_patch(frame_patch)

    return ax



from matplotlib import axes as mpl_axes
import numpy as np

from .calc_aspect_ import calc_aspect


def set_aspect(ax: mpl_axes.Axes, aspect: float) -> None:
    """Adjust the ratio between ylim span length and xlim span length.

    The function calculates the current aspect ratio of the Axes object and
    adjusts its x-axis or y-axis limits to match the desired aspect ratio.
    If the desired aspect ratio is less than the current, the function
    increases the width of the x-axis. If it is greater, the height of the
    y-axis is increased.

    Note that axes limits are only ever widened. Axes widening is performed
    symmetrically.
    """
    cur_aspect = calc_aspect(ax)
    cur_width = np.ptp(ax.get_xlim())
    cur_height = np.ptp(ax.get_ylim())
    x_min, x_max = ax.get_xlim()
    y_min, y_max = ax.get_ylim()

    if aspect == cur_aspect:
        # The current aspect ratio matches the desired one, no action needed
        pass
    elif aspect < cur_aspect:
        # The desired aspect ratio is less than the current,
        # meaning the plot is too tall. We need to increase the width evenly.
        new_width = cur_width / aspect * cur_aspect
        width_extend = (new_width - cur_width) / 2
        ax.set_xlim(x_min - width_extend, x_max + width_extend)
    elif aspect > cur_aspect:
        # The desired aspect ratio is greater than the current,
        # meaning the plot is too wide. We need to increase the height evenly.
        new_height = cur_height * aspect / cur_aspect
        height_extend = (new_height - cur_height) / 2
        ax.set_ylim(y_min - height_extend, y_max + height_extend)
    else:
        assert False, (aspect, cur_aspect)

    new_aspect = calc_aspect(ax)
    assert np.isclose(new_aspect, aspect), (new_aspect, aspect)



from matplotlib import pyplot as plt

from outset._auxlib.set_aspect_ import set_aspect


def test_calc_aspect_square():
    plt.gca().set_ylim(0, 1)
    plt.gca().set_xlim(0, 1)
    set_aspect(plt.gca(), 0.5) == 1
    assert plt.gca().get_xlim() == (-0.5, 1.5)
    assert plt.gca().get_ylim() == (0, 1)

    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(1, 2)
    set_aspect(plt.gca(), 2) == 1
    assert plt.gca().get_xlim() == (0, 1)
    assert plt.gca().get_ylim() == (0.5, 2.5)


def test_calc_aspect_tall():
    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(0, 2)
    set_aspect(plt.gca(), 1)
    assert plt.gca().get_xlim() == (-0.5, 1.5)
    assert plt.gca().get_ylim() == (0, 2)

    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(0, 2)
    set_aspect(plt.gca(), 4)
    assert plt.gca().get_xlim() == (0, 1)
    assert plt.gca().get_ylim() == (-1, 3)

    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(0, 2)
    set_aspect(plt.gca(), 0.5)
    assert plt.gca().get_ylim() == (0, 2)
    assert plt.gca().get_xlim() == (-1.5, 2.5)


def test_calc_aspect_wide():
    plt.gca().set_ylim(0, 1)
    plt.gca().set_xlim(0, 2)
    set_aspect(plt.gca(), 1)
    assert plt.gca().get_ylim() == (-0.5, 1.5)
    assert plt.gca().get_xlim() == (0, 2)

    plt.gca().set_ylim(0, 1)
    plt.gca().set_xlim(0, 2)
    set_aspect(plt.gca(), 1 / 4)
    assert plt.gca().get_ylim() == (0, 1)
    assert plt.gca().get_xlim() == (-1, 3)

    plt.gca().set_ylim(0, 1)
    plt.gca().set_xlim(0, 2)
    set_aspect(plt.gca(), 1 / 0.5)
    assert plt.gca().get_xlim() == (0, 2)
    assert plt.gca().get_ylim() == (-1.5, 2.5)



from outset._auxlib.calc_aspect_ import calc_aspect
from outset._auxlib.equalize_aspect_ import equalize_aspect

from matplotlib import pyplot as plt
import pytest


def test_equalize_aspect():
    # Create a figure with two subplots having different aspect ratios
    fig, (ax1, ax2) = plt.subplots(1, 2)
    ax1.set_xlim(0, 10)
    ax1.set_ylim(0, 20)  # Aspect ratio 2:1
    ax2.set_xlim(0, 10)
    ax2.set_ylim(0, 10)  # Aspect ratio 1:1

    # Apply the equalize_aspect function
    new_aspect = equalize_aspect([ax1, ax2])

    # Calculate aspect ratios after equalization
    aspect_ax1 = calc_aspect(ax1)
    aspect_ax2 = calc_aspect(ax2)

    # Check if both aspect ratios are equal and match the calculated new aspect ratio
    assert aspect_ax1 == pytest.approx(
        new_aspect
    ), "Aspect ratios are not equalized"
    assert aspect_ax2 == pytest.approx(
        new_aspect
    ), "Aspect ratios are not equalized"

    original_aspect1 = 20 / 10
    original_aspect2 = 10 / 10
    expected_new_aspect = (original_aspect1 * original_aspect2) ** 0.5
    assert new_aspect == pytest.approx(
        expected_new_aspect
    ), "New aspect ratio is not the geometric mean of the original aspect ratios"


def test_equalize_aspect_singleton():
    plt.gca().set_xlim(0, 10)
    plt.gca().set_ylim(1, 6)
    xlim_before = plt.gca().get_xlim()
    ylim_before = plt.gca().get_ylim()
    assert equalize_aspect([plt.gca()]) == 0.5
    assert plt.get_xlim() == (0, 10)
    assert plt.get_ylim() == (1, 6)


def test_equalize_aspect_singleton():
    assert equalize_aspect([]) == 1.0



import pandas as pd

from outset._calc_outset_frames import calc_outset_frames

# Sample data for testing
data = pd.DataFrame(
    {"x": [1, 2, 3, 4], "y": [1, 3, 2, 1], "hue": ["A", "B", "A", "B"]}
)


def test_calc_outset_frames():
    res = calc_outset_frames(
        data=data,
        x="x",
        y="y",
        frame_inner_pad=0,
    )
    assert res == [(1, 4, 1, 3)]


def test_calc_outest_frames_with_hue():
    res = calc_outset_frames(
        data=data,
        x="x",
        y="y",
        hue="hue",
        hue_order=["A", "B"],
        frame_inner_pad=0,
    )
    assert res == [(1, 3, 1, 2), (2, 4, 1, 3)]

    res = calc_outset_frames(
        data=data,
        x="x",
        y="y",
        hue="hue",
        hue_order=["B", "A"],
        frame_inner_pad=0,
    )
    assert res == [(2, 4, 1, 3), (1, 3, 1, 2)]

    res = calc_outset_frames(
        data=data,
        x="x",
        y="y",
        hue="hue",
        hue_order=["A"],
        frame_inner_pad=0,
    )
    assert res == [(1, 3, 1, 2)]



import itertools as it
import typing

import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from matplotlib.axes import Axes as mpl_Axes

from .draw_outset_ import draw_outset


def outsetplot(
    data: pd.DataFrame,
    *,
    x: str,
    y: str,
    hue: typing.Optional[str] = None,
    hue_order: typing.Optional[typing.Sequence] = None,
    ax: typing.Optional[mpl_Axes] = None,
    frame_inner_pad: float = 0.1,
    palette: typing.Optional[typing.Sequence] = None,
    **kwargs,
) -> typing.Tuple[mpl_Axes, typing.List[typing.Tuple[float, float]]]:
    """Creates outset annotation(s) containing specified x, y points from a
    pandas DataFrame, potentially within-color coded groups determined by a
    categorical column `hue`.

    Parameters
    ----------
    data : pd.DataFrame
        DataFrame containing the data to be plotted.
    x : str
        Name of the column in `data` to be used for x-axis values.
    y : str
        Name of the column in `data` to be used for y-axis values.
    hue : str, optional
        Name of the categorical column in `data` to produce different-colored annotated subsets.

        If provided, colors are chosen according to palette.
    hue_order : Sequence, optional
        Order to plot the categorical levels in.

        If None, hues are assigned based on hue column sorted order.
     ax : matplotlib.axes.Axes, optional
        Matplotlib Axes object to draw the plot on. If None, the current axes are used.
    frame_inner_pad : float, default 0.1
        How far from data range should rectangular boundary fall?

        Padding is determined relative to axis viewport.
    palette : Sequence, optional
        Color palette for plotting.

        If None, the default seaborn color palette is used. Passing `color`
        kwarg overrides `palette`.
    **kwargs
        Additional keyword arguments forward to `draw_outset`.

    Returns
    -------
    Tuple[matplotlib.axes.Axes, List[Tuple[float, float, float, float]]]
            The matplotlib axes containing the plot.
    """
    if ax is None:
        ax = plt.gca()

    if palette is None:
        palette = sns.color_palette()

    if hue is not None:
        if hue_order is None:
            hue_order = sorted(data[hue].unique())
        groups = data.groupby(hue, sort=False)
        # adapted from https://stackoverflow.com/a/39275799
    else:
        if hue_order is not None:
            raise ValueError("hue_order cannot be specified without hue")
        hue_order = [None]
        groups = [(None, data)]

    palette_lookup = dict(zip(hue_order, it.cycle(palette)))
    frames = dict()
    for hue_value, subset in groups:
        assert len(subset)
        if hue_value not in palette_lookup:
            continue

        xlim = [subset[x].min(), subset[x].max()]
        ylim = [subset[y].min(), subset[y].max()]
        color = palette_lookup[hue_value]
        __, frame = draw_outset(
            xlim,
            ylim,
            ax,
            color=kwargs.get("color", color),
            frame_inner_pad=frame_inner_pad,
            **kwargs,
        )
        frames[hue_value] = frame

    return ax, [frames[hue_value] for hue_value in hue_order]



import typing

from matplotlib import pyplot as plt

from .outsetplot_ import outsetplot


def calc_outset_frames(
    *args, **kwargs
) -> typing.List[typing.Tuple[float, float, float, float]]:
    if "ax" in kwargs:
        raise ValueError("as cannot be passed to _calc_outset_frames")
    dummy_fig = plt.figure()
    dummy_ax = dummy_fig.add_subplot(111)
    _ax, res = outsetplot(*args, ax=dummy_ax, **kwargs)
    plt.close(dummy_fig)

    return res



from matplotlib import axes as mpl_axes
import numpy as np


def calc_aspect(ax: mpl_axes.Axes) -> float:
    """Calculate the ratio between ylim span length and xlim span length."""
    return np.ptp(ax.get_ylim()) / np.ptp(ax.get_xlim())



from matplotlib import pyplot as plt

from outset._auxlib.calc_aspect_ import calc_aspect


def test_calc_aspect_square():
    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(0, 1)
    assert calc_aspect(plt.gca()) == 1


def test_calc_aspect_tall():
    plt.gca().set_xlim(0, 1)
    plt.gca().set_ylim(0, 2)
    assert calc_aspect(plt.gca()) == 2


def test_calc_aspect_wide():
    plt.gca().set_xlim(-1, 3)
    plt.gca().set_ylim(0, 2)
    assert calc_aspect(plt.gca()) == 0.5
